import nonebot
from apscheduler.triggers.cron import CronTrigger
from nonebot import scheduler
from nonebot.log import logger

from .utils import User, read_config, safe_send, update_config

index = 0

@scheduler.scheduled_job(CronTrigger(second='*/10'), id='live_sched')
@logger.catch
async def live_sched():
    config = await read_config()
    ups = config['status']
    
    uid_list = config['live']['uid_list']
    global index
    if not uid_list:
        return
    if index >= len(uid_list):
        uid = uid_list[0]
        index = 1
    else:
        uid = uid_list[index]
        index += 1

    old_status = ups[uid]
    user = User(uid)
    user_info = (await user.get_info())['live_room']
    name = config['uid'][uid]['name'] # 直接从配置文件读取名字
    logger.debug(f'爬取直播 [{index:03}] {name}({uid})')
    new_status = user_info['liveStatus']
    if new_status != old_status:
        config = await read_config()
        config['status'][uid] = new_status
        await update_config(config)

        if new_status:
            # name = (await user.get_info())['name'] # 获取昵称应转移至配置文件
            live_msg = f"{name} 开播啦！\n\n{user_info['title']}\n传送门→{user_info['url']}\n[CQ:image,file={user_info['cover']}]"
            groups = config["uid"][uid]["groups"]
            for group_id, bot_id in groups.items():
                if config["groups"][group_id]['uid'][uid]["live"]:
                    bots = nonebot.get_bots()
                    bot = bots[bot_id]
                    if config['groups'][group_id]['uid'][uid]['at']:
                        await safe_send(bot, 'group', group_id, "[CQ:at,qq=all] "+live_msg)
                    else:
                        await safe_send(bot, 'group', group_id, live_msg)
            users = config["uid"][uid]["users"]
            for user_id, bot_id in users.items():
                if config["users"][user_id]['uid'][uid]["live"]:
                    bots = nonebot.get_bots()
                    bot = bots[bot_id]
                    await safe_send(bot, 'private', user_id, live_msg)



import asyncio
import base64
import json
import os
import platform
import subprocess
import traceback
from datetime import datetime
from os import path

import httpx
import nonebot
import psutil
from nonebot.adapters.cqhttp import Bot
from nonebot.log import logger
from nonebot import get_driver

os.environ['PYPPETEER_DOWNLOAD_HOST'] = 'http://npm.taobao.org/mirrors'
# os.environ['PYPPETEER_DOWNLOAD_HOST'] = 'http://storage.googleapis.com'
from pyppeteer import launch
from pyppeteer.chromium_downloader import check_chromium, download_chromium

if not check_chromium():
    download_chromium()

class Dynamic():
    def __init__(self, dynamic):
        # self.origin = json.loads(self.card['origin'])
        self.dynamic = dynamic
        # self.card = json.loads(dynamic['card'])
        # self.dynamic['card'] = self.card
        self.type = dynamic['desc']['type']
        self.id = dynamic['desc']['dynamic_id']
        self.url = "https://t.bilibili.com/" + str(self.id)
        self.time = dynamic['desc']['timestamp']
        # self.origin_id = dynamic['desc']['orig_dy_id']
        self.name = dynamic['desc']['user_profile']['info']['uname']
        self.uid = dynamic['desc']['user_profile']['info']['uid']
        self.img_name = str(self.uid) + str(self.time) + '.png'
        self.img_path = get_path(self.img_name)

    async def format(self):
        if self.type == 1:
            self.message = f"{self.name}转发了一条动态：\n\n传送门→" + self.url + "[CQ:image,file=" + self.image + "]\n"
            return self.message
        elif self.type == 8:
            bv_url = 'https://www.bilibili.com/video/' + self.dynamic['desc']['bvid']
            self.message = f"{self.name}发布了新投稿\n\n传送门→" + bv_url + "[CQ:image,file=" + self.image + "]\n"
        elif self.type == 16:
            self.message = f"{self.name}发布了短视频\n\n传送门→" + self.url + "[CQ:image,file=" + self.image + "]\n"
        elif self.type == 64:
            self.message = f"{self.name}发布了新专栏\n\n传送门→" + self.url + "[CQ:image,file=" + self.image + "]\n"
        elif self.type == 256:
            self.message = f"{self.name}发布了新音频\n\n传送门→" + self.url + "[CQ:image,file=" + self.image + "]\n"
        else:
            self.message = f"{self.name}发布了新动态\n\n传送门→" + self.url + "[CQ:image,file=" + self.image + "]\n"

    async def get_screenshot(self):
        if path.isfile(self.img_path):
            return
        browser = await launch(args=['--no-sandbox'])
        page = await browser.newPage()
        for _ in range(3):
            try:
                await page.goto(self.url, waitUntil="networkidle0")
                # await page.waitForNavigation()
                # await page.waitFor(1000)
                await page.setViewport(viewport={'width': 1920, 'height': 1080})
                # card = await page.waitForSelector(".card")
                card = await page.querySelector(".card")
                clip = await card.boundingBox()
                bar = await page.querySelector(".text-bar")
                bar_bound = await bar.boundingBox()
                clip['height'] = bar_bound['y'] - clip['y']
                await page.screenshot({'path': self.img_path, 'clip': clip})
                break
            except:
                logger.error(traceback.format_exc())
                await asyncio.sleep(0.1)
        await page.close()
        await browser.close()
    
    async def encode(self):
        """将图片转为base64码"""
        with open(self.img_path, 'rb') as f:
            self.image = "base64://" + base64.b64encode(f.read()).decode('utf-8')
            return self.image
    
    async def get_path(self):
        self.image = "file:///" + self.img_path
        return self.image


class User():
    def __init__(self, uid):
        self.uid = str(uid)
    
    async def get_info(self):
        url = f'https://api.bilibili.com/x/space/acc/info?mid={self.uid}'
        return (await Get(url))['data']

    async def get_dynamic(self):
        # need_top: {1: 带置顶, 0: 不带置顶}
        url = f'https://api.vc.bilibili.com/dynamic_svr/v1/dynamic_svr/space_history?host_uid={self.uid}&offset_dynamic_id=0&need_top=0'
        return (await Get(url))['data']
    
    async def get_live_info(self):
        url = f'https://api.live.bilibili.com/room/v1/Room/getRoomInfoOld?mid={self.uid}'
        return (await Get(url))['data']


async def Get(url):
    DEFAULT_HEADERS = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
                  "Chrome/79.0.3945.130 Safari/537.36",
    "Referer": "https://www.bilibili.com/"
    }
    async with httpx.AsyncClient() as client:
        r = await client.get(url, headers=DEFAULT_HEADERS)
    r.encoding = 'utf-8'
    return r.json()


async def read_config():
    """读取用户注册信息"""
    try:
        with open(get_path('config.json'), encoding='utf-8-sig') as f:
            config = json.loads(f.read())
    except FileNotFoundError:
        config = get_new_config()
    return config


def get_new_config():
    return {"status": {}, "uid": {}, "groups": {}, "users": {}, "dynamic": {"uid_list": [], "index": 0}, 'live': {'uid_list': [], 'index': 0}}


async def update_config(config):
    """更新注册信息"""
    with open(get_path('config.json'), 'w', encoding='utf-8') as f:
        f.write(json.dumps(config, ensure_ascii=False, indent=4))


async def backup_config(config):
    # backup_name = f"config{datetime.now().strftime('%Y.%m.%d %H-%M-%S')}.json"
    backup_name = f"config{int(datetime.now().timestamp())}.json"
    with open(get_path(backup_name), 'w', encoding='utf-8') as f:
        f.write(json.dumps(config, ensure_ascii=False, indent=4))


def get_path(name):
    """获取数据文件绝对路径"""
    config = get_driver().config
    if config.haruka_dir:
        dir_path = path.abspath(config.haruka_dir)
    else:
        src_path = path.dirname(path.abspath(__file__))
        dir_path = path.join(src_path, 'data')
    f_path = path.join(dir_path, name)
    return f_path


async def safe_send(bot: Bot, send_type, type_id, message):
    """发送出现错误时, 尝试重新发送, 并捕获异常且不会中断运行"""
    for i in range(3):
        try:
            message_id = await bot.call_api('send_'+send_type+'_msg', **{
                'message': message,
                'user_id' if send_type == 'private' else 'group_id': type_id
                })
            return message_id
        except:
            logger.error(traceback.format_exc())
            if i == 2:
                bot = await restart(bot)
                message = '检测到推送出现异常，已尝试自动重启，如仍有问题请向机器人管理员反馈'
                message_id = await bot.call_api('send_'+send_type+'_msg', **{
                    'message': message,
                    'user_id' if send_type == 'private' else 'group_id': type_id
                    })
                return message_id
            await asyncio.sleep(0.1)


async def restart(bot: Bot):
    cqhttp_dir = (await bot.get_version_info())['coolq_directory']
    pids = psutil.process_iter()
    for pid in pids:
        try:
            if 'go-cqhttp' in pid.name() and \
                pid.exe() == path.join(cqhttp_dir, pid.name()):
                subprocess.Popen(
                    pid.exe(), 
                    cwd=pid.cwd(), 
                    # stdout=subprocess.PIPE, 
                    stderr=None if platform.system()=='Windows' else subprocess.DEVNULL, 
                    creationflags=subprocess.CREATE_NEW_CONSOLE if platform.system()=='Windows' else 0
                    )
                pid.terminate()
                while True:
                    await asyncio.sleep(1)
                    bots = nonebot.get_bots()
                    if bot.self_id in bots:
                        return bots[bot.self_id]
        except psutil.ZombieProcess:
            continue


# bot 启动时检查 src\data\haruka_bot\ 目录是否存在
if not path.isdir(get_path('')):
    os.makedirs(get_path(''))



from nonebot import on_command
from nonebot.rule import to_me
from nonebot.adapters.cqhttp import Bot, Event
from pydantic import main


func_list = """DD机目前支持的功能有：

  主播列表
  修复配置
  开启权限
  关闭权限
  添加主播 uid
  删除主播 uid
  开启动态 uid
  关闭动态 uid
  开启直播 uid
  关闭直播 uid
  开启at uid
  关闭at uid


命令中的uid需要替换为对应主播的uid，注意是uid不是直播间id

群聊默认开启权限，只有管理员或机器人主人才能触发指令

所有群聊/私聊的推送都是分开的，在哪里添加就只会在哪里推送
"""


help = on_command('帮助', rule=to_me(), priority=5)

@help.handle()
async def _(bot: Bot, event: Event, state: dict):
    await help.finish(func_list)




from collections import Counter
from nonebot import on_command
from nonebot.rule import to_me
from nonebot.adapters.cqhttp import Bot, Event
from nonebot.permission import GROUP_ADMIN, PRIVATE_FRIEND, SUPERUSER, GROUP_OWNER
from .utils import get_new_config, read_config, User, update_config, backup_config


async def permission_check(bot: Bot, event: Event, state: dict):
    config = await read_config()
    if event.detail_type == 'group':
        group_id = str(event.group_id)
        if True if group_id not in config['groups'] else config['groups'][group_id]['admin']:
            is_admin = await GROUP_ADMIN(bot, event) or await GROUP_OWNER(bot, event) or await PRIVATE_FRIEND(bot, event) or await SUPERUSER(bot, event)
            if is_admin:
                return True
            else:
                # await bot.send(event, '权限不足，无法使用')
                return False
        else:
            return True
    else:
        return True # 私聊默认返回 True


add_uid = on_command('添加主播', rule=to_me() & permission_check, priority=5)

@add_uid.handle()
async def get_args(bot: Bot, event: Event, state: dict):
    args = str(event.message).strip()
    if args:
        state['uid'] = args

@add_uid.got('uid', prompt='请输入主播的uid')
async def _(bot: Bot, event: Event, state: dict):
    uid = state['uid']
    config = await read_config()

    if uid not in config["status"]: # uid不在配置文件就创建一个
        user = User(uid)
        name = ''
        try: # 应该改uid有效检测逻辑
            user_info = await user.get_info()
            name = user_info["name"]
        except:
            await add_uid.finish("请输入有效的uid")
        config['status'][uid] = 0
        config['uid'][uid] = {'groups': {}, 'users': {}, 'dynamic': 0, 'live': 0, 'name': name}
        config['dynamic']['uid_list'].append(uid) # 主播uid添加至动态列表
        config['live']['uid_list'].append(uid) # 主播uid添加至直播列表
    else:
        name = config['uid'][uid]['name']
        if uid not in config['dynamic']['uid_list']: 
            config['dynamic']['uid_list'].append(uid)
        if uid not in config['live']['uid_list']:
            config['live']['uid_list'].append(uid)

    if event.detail_type == "group": # 检测是否群消息
        group_id = str(event.group_id)
        if group_id not in config["uid"][uid]["groups"]:
            config["uid"][uid]["groups"][group_id] = event.self_id
        else:
            await add_uid.finish(f"您已将{name}（{uid}）添加至该群，请勿重复添加")
        if group_id in config["groups"]:
            config["groups"][group_id]["uid"][uid] = {"live": True, "dynamic": True, 'at': False}
        else:
            config["groups"][group_id] = {"uid": {uid: {"live": True, "dynamic": True, 'at': False}}, 'admin': True}
        config['uid'][uid]['dynamic'] += 1
        config['uid'][uid]['live'] += 1
        await update_config(config)
        await add_uid.finish(f"已将{name}（{uid}）添加至该群")

    elif event.detail_type == "private": # 检测是否私聊消息
        user_id = str(event.user_id)
        if user_id not in config["uid"][uid]["users"]:
            config["uid"][uid]["users"][user_id] = event.self_id
        else:
            await add_uid.finish(f"您已添加{name}（{uid}），请勿重复添加")
        if user_id in config["users"]:
            config["users"][user_id]["uid"][uid] = {"live": True, "dynamic": True}
        else:
            config["users"][user_id] = {"uid": {uid: {"live": True, "dynamic": True}}}
        config['uid'][uid]['dynamic'] += 1
        config['uid'][uid]['live'] += 1
        await update_config(config)
        await add_uid.finish(f"已添加{name}（{uid}）")


delete_uid = on_command('删除主播', rule=to_me() & permission_check, priority=5)

@delete_uid.handle()
async def get_args(bot: Bot, event: Event, state: dict):
    args = str(event.message).strip()
    if args:
        state['uid'] = args

@delete_uid.got('uid', prompt='请输入主播的uid')
async def _(bot: Bot, event: Event, state: dict):
    uid = state['uid']
    config = await read_config()

    try:
        name = config['uid'][uid]['name']
    except KeyError:
        await delete_uid.finish("删除失败，uid不存在")
    if event.detail_type == 'group':
        group_id = str(event.group_id)
        try:
            if config['groups'][group_id]['uid'][uid]['dynamic']:
                config['uid'][uid]['dynamic'] -= 1
            if config['groups'][group_id]['uid'][uid]['live']:
                config['uid'][uid]['live'] -= 1
            del config['groups'][group_id]['uid'][uid]
            del config['uid'][uid]['groups'][group_id]
            # 如果用户没有关注则删除用户
            if config['groups'][group_id]['uid'] == {} and config['groups'][group_id]['admin']:
                del config['groups'][group_id]
        except KeyError:
            await delete_uid.finish("删除失败，uid不存在")
    elif event.detail_type == 'private':
        user_id = str(event.user_id)
        try:
            if config['users'][user_id]['uid'][uid]['dynamic']:
                config['uid'][uid]['dynamic'] -= 1
            if config['users'][user_id]['uid'][uid]['live']:
                config['uid'][uid]['live'] -= 1
            del config['users'][user_id]['uid'][uid]
            del config['uid'][uid]['users'][user_id]
            # 如果用户没有关注则删除用户
            if config['users'][user_id]['uid'] == {}:
                del config['users'][user_id]
        except KeyError:
            await delete_uid.finish("删除失败，uid不存在")
    # 如果无人再订阅动态，就从动态列表中移除
    if config['uid'][uid]['dynamic'] == 0 and uid in config['dynamic']['uid_list']:
        config['dynamic']['uid_list'].remove(uid)
    # 如果无人再订阅直播，就从直播列表中移除
    if config['uid'][uid]['live'] == 0 and uid in config['live']['uid_list']:
        config['live']['uid_list'].remove(uid)
    # 如果没人订阅该主播，则将该主播彻底删除
    if config['uid'][uid]['groups'] == {} and config['uid'][uid]['users'] == {}:
        del config['uid'][uid]
        del config['status'][uid]
    
    # user = User(uid)
    # name = (await user.get_info())['name']
    await update_config(config)
    await delete_uid.finish(f"已删除 {name}（{uid}）")


list_uid = on_command('主播列表', rule=to_me() & permission_check, priority=5)

@list_uid.handle()
async def _(bot: Bot, event: Event, state: dict):
    config = await read_config()
    if event.detail_type == 'group':
        group_id = str(event.group_id)
        try:
            uid_list = config['groups'][group_id]['uid']
        except KeyError:
            uid_list = {}
        message = "以下为该群的订阅列表，可发送“删除主播 uid”进行删除\n\n"
    elif event.detail_type == 'private':
        user_id = str(event.user_id)
        try:
            uid_list = config['users'][user_id]['uid']
        except KeyError:
            uid_list = {}
        message = "以下为您的订阅列表，可发送“删除主播 uid”进行删除\n\n"
    for uid, status in uid_list.items():
        name = config['uid'][uid]['name']
        message += f"【{name}】"
        message += f"直播推送：{'开' if status['live'] else '关'}，"
        message += f"动态推送：{'开' if status['dynamic'] else '关'}"
        message += f"（{uid}）\n"
    await list_uid.send(message=message)


dynamic_on = on_command('开启动态', rule=to_me() & permission_check, priority=5)

@dynamic_on.handle()
async def handle_first_recive(bot: Bot, event: Event, state: dict):
    args = str(event.message).strip()
    if args:
        state['uid'] = args

@dynamic_on.got('uid', prompt='请输入主播的uid')
async def _(bot: Bot, event: Event, state: dict):
    uid = state['uid']
    config = await read_config()

    if event.detail_type == 'group':
        group_id = str(event.group_id)
        try:
            if config['groups'][group_id]['uid'][uid]['dynamic'] == True:
                await dynamic_on.finish('请勿重复开启动态推送')
            config['groups'][group_id]['uid'][uid]['dynamic'] = True
            config['uid'][uid]['dynamic'] += 1
        except KeyError:
            await dynamic_on.finish("开启失败，uid不存在")
    elif event.detail_type == 'private':
        user_id = str(event.user_id)
        try:
            if config['users'][user_id]['uid'][uid]['dynamic'] == True:
                await dynamic_on.finish('请勿重复开启动态推送')
            config['users'][user_id]['uid'][uid]['dynamic'] = True
            config['uid'][uid]['dynamic'] += 1
        except KeyError:
            await dynamic_on.finish("开启失败，uid不存在")
    # 如果是第一个开启的，就添加至动态推送列表
    if config['uid'][uid]['dynamic'] == 1:
        config['dynamic']['uid_list'].append(uid)
    name = config['uid'][uid]['name']
    # user = User(uid)
    # name = (await user.get_info())['name']
    await update_config(config)
    await dynamic_on.finish(f"已开启 {name}（{uid}）的动态推送")


dynamic_off = on_command('关闭动态', rule=to_me() & permission_check, priority=5)

@dynamic_off.handle()
async def handle_first_recive(bot: Bot, event: Event, state: dict):
    args = str(event.message).strip()
    if args:
        state['uid'] = args

@dynamic_off.got('uid', prompt='请输入主播的uid')
async def _(bot: Bot, event: Event, state: dict):
    uid = state['uid']
    config = await read_config()

    if event.detail_type == 'group':
        group_id = str(event.group_id)
        try:
            if config['groups'][group_id]['uid'][uid]['dynamic'] == False:
                await dynamic_off.finish('请勿重复关闭动态推送')
            config['groups'][group_id]['uid'][uid]['dynamic'] = False
            config['uid'][uid]['dynamic'] -= 1
        except KeyError:
            await dynamic_off.finish("关闭失败，uid不存在")
    elif event.detail_type == 'private':
        user_id = str(event.user_id)
        try:
            if config['users'][user_id]['uid'][uid]['dynamic'] == False:
                await dynamic_off.finish('请勿重复关闭动态推送')
            config['users'][user_id]['uid'][uid]['dynamic'] = False
            config['uid'][uid]['dynamic'] -= 1
        except KeyError:
            await dynamic_off.finish("关闭失败，uid不存在")
    # 如果无人再订阅动态，就从动态列表中移除
    if config['uid'][uid]['dynamic'] == 0:
        config['dynamic']['uid_list'].remove(uid)
    name = config['uid'][uid]['name']
    # user = User(uid)
    # name = (await user.get_info())['name']
    await update_config(config)
    await dynamic_off.finish(f"已关闭 {name}（{uid}）的动态推送")


live_on = on_command('开启直播', rule=to_me() & permission_check, priority=5)

@live_on.handle()
async def handle_first_recive(bot: Bot, event: Event, state: dict):
    args = str(event.message).strip()
    if args:
        state['uid'] = args

@live_on.got('uid', prompt='请输入主播的uid')
async def _(bot: Bot, event: Event, state: dict):
    uid = state['uid']
    config = await read_config()
    
    if event.detail_type == 'group':
        group_id = str(event.group_id)
        try:
            if config['groups'][group_id]['uid'][uid]['live']:
                await live_on.finish('请勿重复开启直播推送')
            config['groups'][group_id]['uid'][uid]['live'] = True
            config['uid'][uid]['live'] += 1
        except KeyError:
            await live_on.finish("开启失败，uid不存在")
    elif event.detail_type == 'private':
        user_id = str(event.user_id)
        try:
            if config['users'][user_id]['uid'][uid]['live']:
                await live_on.finish('请勿重复开启直播推送')
            config['users'][user_id]['uid'][uid]['live'] = True
            config['uid'][uid]['live'] += 1
        except KeyError:
            await live_on.finish("开启失败，uid不存在")
    # 如果是第一个开启的，就添加至直播推送列表
    if config['uid'][uid]['live'] == 1:
        config['live']['uid_list'].append(uid)
    name = config['uid'][uid]['name']
    # user = User(uid)
    # name = (await user.get_info())['name']
    await update_config(config)
    await live_on.finish(f"已开启 {name}（{uid}）的直播推送")
    

live_off = on_command('关闭直播', rule=to_me() & permission_check, priority=5)

@live_off.handle()
async def handle_first_recive(bot: Bot, event: Event, state: dict):
    args = str(event.message).strip()
    if args:
        state['uid'] = args

@live_off.got('uid', prompt='请输入主播的uid')
async def _(bot: Bot, event: Event, state: dict):
    uid = state['uid']
    config = await read_config()

    if event.detail_type == 'group':
        group_id = str(event.group_id)
        try:
            if not config['groups'][group_id]['uid'][uid]['live']:
                await live_off.finish('请勿重复关闭直播推送')
            config['groups'][group_id]['uid'][uid]['live'] = False
            config['uid'][uid]['live'] -= 1
        except KeyError:
            await live_off.finish("关闭失败，uid不存在")
    elif event.detail_type == 'private':
        user_id = str(event.user_id)
        try:
            if not config['users'][user_id]['uid'][uid]['live']:
                await live_off.finish('请勿重复关闭直播推送')
            config['users'][user_id]['uid'][uid]['live'] = False
            config['uid'][uid]['live'] -= 1
        except KeyError:
            await live_off.finish("关闭失败，uid不存在")
    # 如果无人再订阅动态，就从直播列表中移除
    if config['uid'][uid]['live'] == 0:
        config['live']['uid_list'].remove(uid)
    name = config['uid'][uid]['name']
    # user = User(uid)
    # name = (await user.get_info())['name']
    await update_config(config)
    await live_off.finish(f"已关闭 {name}（{uid}）的直播推送")


at_on = on_command('开启at', rule=to_me(), 
    permission=GROUP_OWNER | GROUP_ADMIN | PRIVATE_FRIEND | SUPERUSER, 
    priority=5)

@at_on.handle()
async def handle_first_recive(bot: Bot, event: Event, state: dict):
    args = str(event.message).strip()
    if args:
        state['uid'] = args

@at_on.got('uid', prompt='请输入主播的uid')
async def _(bot: Bot, event: Event, state: dict):
    if event.detail_type != 'group':
        await at_on.finish("只有群里才能开启@全体")
    uid = state['uid']
    config = await read_config()

    group_id = event.group_id
    if uid not in config['groups'][str(group_id)]['uid']:
        await at_on.finish("开启失败，uid不存在")
    if config['groups'][str(group_id)]['uid'][uid]['at']:
        await at_on.finish("请勿重复开启@全员")
    config['groups'][str(group_id)]['uid'][uid]['at'] = True
    await update_config(config)
    name = config['uid'][uid]['name']
    await at_on.finish(f"已开启 {name}（{uid}）的 @全体成员")


at_off = on_command('关闭at', rule=to_me(), 
    permission=GROUP_OWNER | GROUP_ADMIN | PRIVATE_FRIEND | SUPERUSER, 
    priority=5)

@at_off.handle()
async def handle_first_recive(bot: Bot, event: Event, state: dict):
    args = str(event.message).strip()
    if args:
        state['uid'] = args

@at_off.got('uid', prompt='请输入主播的uid')
async def _(bot: Bot, event: Event, state: dict):
    if event.detail_type != 'group':
        await at_off.finish("只有群里才能关闭@全体")
    uid = state['uid']
    config = await read_config()

    group_id = event.group_id
    if uid not in config['groups'][str(group_id)]['uid']:
        await at_off.finish("关闭失败，uid不存在")
    if not config['groups'][str(group_id)]['uid'][uid]['at']:
        await at_off.finish("请勿重复关闭@全员")
    config['groups'][str(group_id)]['uid'][uid]['at'] = False
    await update_config(config)
    name = config['uid'][uid]['name']
    await at_off.finish(f"已关闭 {name}（{uid}）的 @全体成员")


permission_on = on_command('开启权限', rule=to_me(), 
    permission=GROUP_OWNER | GROUP_ADMIN | PRIVATE_FRIEND | SUPERUSER, 
    priority=5)

@permission_on.handle()
async def _(bot: Bot, event: Event, state: dict):
    if event.detail_type != 'group':
        await permission_on.finish("只有群里才能设置权限")
    config = await read_config()
    group_id = str(event.group_id)

    if group_id not in config['groups'] or config['groups'][group_id]['admin']:
        await permission_on.finish("请勿重复开启权限")
    else:
        config['groups'][group_id]['admin'] = True

    await update_config(config)
    await permission_on.finish(f"已开启权限限制，只有管理员才能触发指令")


permission_off = on_command('关闭权限', rule=to_me(), 
    permission=GROUP_OWNER | GROUP_ADMIN | PRIVATE_FRIEND | SUPERUSER, 
    priority=5)

@permission_off.handle()
async def _(bot: Bot, event: Event, state: dict):
    if event.detail_type != 'group':
        await permission_off.finish("只有群里才能设置权限")
    config = await read_config()
    group_id = str(event.group_id)

    if group_id not in config['groups']:
        config['groups'][group_id] = {'uid': {}, 'admin': False}
    elif not config['groups'][group_id]['admin']:
        await permission_off.finish("请勿重复关闭权限")
    else:
        config['groups'][group_id]['admin'] = False

    await update_config(config)
    await permission_off.finish(f"已关闭权限限制，所有人都能触发指令")


fix_config = on_command('修复配置', rule=to_me(), 
    permission=GROUP_OWNER | GROUP_ADMIN | PRIVATE_FRIEND | SUPERUSER, 
    priority=5)

@fix_config.handle()
async def _(bot: Bot, event: Event, state: dict):
    config = await read_config()
    new_config = get_new_config()
    dy_counter = Counter() # 动态推送开启用户数统计
    live_counter = Counter() # 直播推送开启用户数统计
    
    for config_type in ['groups', 'users']:
        for type_id, type_config in config[config_type].items():
            uids = type_config['uid']
            if uids == {}:
                continue
            new_config[config_type][type_id] = {'uid': {}}
            if config_type == 'groups':
                new_config[config_type][type_id]['admin'] = type_config.get('admin', True)
            for uid, status in uids.items():
                new_config[config_type][type_id]['uid'][uid] = {'live': False, 'dynamic': False}
                new_status = new_config[config_type][type_id]['uid'][uid]
                if 'live' in status and status['live'] or status['live_reminder']:
                    live_counter[uid] += 1
                    new_status['live'] = True
                if status['dynamic']:
                    dy_counter[uid] += 1
                    new_status['dynamic'] = True
                if config_type == 'groups':
                    new_status['at'] = status['at']
                
                if uid not in new_config['uid']:
                    new_config['uid'][uid] = {
                        'groups': {},
                        'users': {},
                        'live': 0,
                        'dynamic': 0,
                        'name': config['uid'][uid]['name']
                        }
                if new_status['live'] or new_status['dynamic']:
                    new_config['uid'][uid][config_type][type_id] = str(config['uid'][uid][config_type][type_id])
                
    for uid, sub_num in dy_counter.items():
        new_config['uid'][uid]['dynamic'] = sub_num
    for uid, sub_num in live_counter.items():
        new_config['uid'][uid]['live'] = sub_num
        new_config['status'][uid] = config['status'].get(uid, 0)
    
    new_config['dynamic'] = {'uid_list': list(dy_counter)}
    new_config['live'] = {'uid_list': list(config['status'])}
    
    await backup_config(config)
    await update_config(new_config)
    await fix_config.finish('修复完成')
    


import os
from datetime import datetime, timedelta

import nonebot
from nonebot import scheduler
from nonebot.log import logger
from apscheduler.triggers.cron import CronTrigger

from .utils import Dynamic, User, read_config, safe_send

last_time = {}
index = 0

@scheduler.scheduled_job(CronTrigger(second='*/10'), id='dynamic_sched')
@logger.catch
async def dy_sched():
    config = await read_config()
    ups = config['dynamic']['uid_list']

    uid_list = ups
    global index
    if not uid_list:
        return
    if index >= len(uid_list):
        uid = uid_list[0]
        index = 1
    else:
        uid = uid_list[index]
        index += 1

    name = config['uid'][uid]['name'] # 直接从配置文件读取名字
    logger.debug(f'爬取动态 [{index:03}] {name}({uid})')
    user = User(uid)
    dynamics = (await user.get_dynamic()).get('cards', []) # 获取最近十二条动态
    # config['uid'][uid]['name'] = dynamics[0]['desc']['user_profile']['info']['uname']
    # await update_config(config)

    if len(dynamics) == 0: # 没有发过动态或者动态全删的直接结束
        return

    if uid not in last_time: # 没有爬取过这位主播就把最新一条动态时间为 last_time
        dynamic = Dynamic(dynamics[0])
        last_time[uid] = dynamic.time
        return
    
    for dynamic in dynamics[4::-1]: # 从旧到新取最近5条动态
        dynamic = Dynamic(dynamic)
        if dynamic.time > last_time[uid] and dynamic.time > datetime.now().timestamp() - timedelta(minutes=10).seconds:
            await dynamic.get_screenshot()
            await dynamic.encode()
            os.remove(dynamic.img_path)
            await dynamic.format()
            for group_id, bot_id in config["uid"][uid]["groups"].items():
                if config["groups"][group_id]['uid'][uid]["dynamic"]:
                    bots = nonebot.get_bots()
                    bot = bots[bot_id]
                    await safe_send(bot, 'group', group_id, dynamic.message)
            for user_id, bot_id in config["uid"][uid]["users"].items():
                if config["users"][user_id]['uid'][uid]["dynamic"]:
                    bots = nonebot.get_bots()
                    bot = bots[bot_id]
                    await safe_send(bot, 'private', user_id, dynamic.message)
            last_time[uid] = dynamic.time



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src\\plugins'}

packages = \
['haruka_bot']

package_data = \
{'': ['*']}

install_requires = \
['nonebot2[scheduler]>=2.0.0-alpha.4,<3.0.0',
 'psutil>=5.7.3,<6.0.0',
 'pyppeteer>=0.2.2,<0.3.0',
 'qrcode[pil]>=6.1,<7.0']

setup_kwargs = {
    'name': 'haruka-bot',
    'version': '1.0.2',
    'description': 'Push dynamics and live informations from bilibili to QQ. Based on nonebot2.',
    'long_description': '  \n  \n<div align="center">\n  <img src="logo.png" width="200" height="200" alt="logo">\n  \n# HarukaBot\n  \n[![VERSION](https://img.shields.io/github/v/release/SK-415/HarukaBot )](https://github.com/SK-415/HarukaBot/releases)\n[![time tracker](https://wakatime.com/badge/github/SK-415/HarukaBot.svg )](https://wakatime.com/badge/github/SK-415/HarukaBot)\n[![STARS](https://img.shields.io/github/stars/SK-415/HarukaBot )](https://github.com/SK-415/HarukaBot/stargazers)\n[![qq group](https://img.shields.io/badge/QQ%E7%BE%A4-629574472-orange )](https://jq.qq.com/?_wv=1027&k=sHPbCRAd)\n[![LICENSE](https://img.shields.io/github/license/SK-415/HarukaBot )](https://github.com/SK-415/HarukaBot/blob/master/LICENSE)\n  \n**由于学业原因, HarukaBot 将停更一段时间, 等我成绩恢复稳定后会继续更新.**  \n**虽然暂时不更新新功能, 但是在此期间我依然会继续完善文档同时在[QQ群](https://jq.qq.com/?_wv=1027&k=DveS3XKI )答疑.**\n  \n</div>\n  \n##  目录\n  \n  \n- [简介](#简介 )\n- [功能介绍](#功能介绍 )\n- [已知问题](#已知问题 )\n  - [推送延迟](#推送延迟 )\n  - [动态推送失效](#动态推送失效 )\n- [部署指南](#部署指南 )\n  - [部署 go-cqhttp](#部署-go-cqhttp )\n  - [部署 HarukaBot](#部署-harukabot )\n    - [方法一 手动安装 (较为复杂全平台通用)](#方法一-手动安装-较为复杂全平台通用 )\n    - [方法二 插件广场安装 (适用于 `NoneBot2` 用户)](#方法二-插件广场安装-适用于-nonebot2-用户 )\n- [支持作者](#支持作者 )\n  \n##  简介\n  \n  \n一款将哔哩哔哩UP主的直播与动态信息推送至QQ的机器人. 基于 [`NoneBot2`](https://github.com/nonebot/nonebot2 ) 开发, 前身为 [`dd-bot`](https://github.com/SK-415/dd-bot ) .\n  \n项目名称来源于B站 [@白神遥Haruka](https://space.bilibili.com/477332594 )\n  \nlogo 画师: [秦无](https://space.bilibili.com/4668826 )\n  \nHarukaBot 致力于为B站UP主们提供一个开源免费的粉丝群推送方案. 极大的减轻管理员负担, 不会再遇到突击无人推送的尴尬情况. 同时还能将B博动态截图转发至粉丝群, 活跃群内话题.\n  \n> 介于作者技术力低下, HarukaBot 的体验可能并不是很好. 如果使用中有任何意见或者建议都欢迎提出, 我会努力去完善它. \n  \n##  功能介绍\n  \n  \n**以下仅为功能介绍, 并非实际命令名称. 具体命令向 bot 发送 `帮助` 查看, 群里要在所有命令前需加上@**\n  \nHarukaBot 专注于订阅B站UP主们的动态与开播提醒, 并转发至QQ群/好友.\n  \n同时 HarukaBot 针对粉丝群中的推送场景进行了若干优化: \n  \n- **权限开关**: 指定某个群只有管理员才能触发指令\n  \n- **@全体开关**: 指定群里某位订阅的主播开播推送带@全体\n  \n- **动态/直播开关**: 可以自由设置每位主播是否推送直播/动态\n  \n- **订阅列表**: 每个群/好友的订阅都是分开来的\n  \n- **多端推送**: 受限于一个QQ号一天只能@十次全体成员, 因此对于粉丝群多的UP来说一个 bot 的次数完全不够推送. 因此一台 HarukaBot 支持同时连接多个QQ, 分别向不同的群/好友同时推送\n  \n##  已知问题\n  \n  \n###  推送延迟\n  \n  \n受限于B站对API爬取频率限制, 目前 HarukaBot 会将所有UP主排成一列, 每隔十秒检查一位. 因此如果 HarukaBot 订阅了 x 位UP主最高延迟就是 10x 秒.\n  \n> 虽然 HarukaBot 目前的推送延迟对于粉丝群来说是足够了, 但是很显然对于广大dd们来说并不友好, 随便订阅30+个主播延迟就能超过5分钟. \n> 因此, 未来的 HarukaBot v2 版中计划支持绑定B站账号, 将摆脱订阅数量越多推送越慢的窘境.\n  \n###  动态推送失效\n  \n  \n在早晨约两点到八点期间, 部分服务器出现动态获取 api 失效的现象, 具体原因不明, 预计在 v2 中通过登录改善.\n  \n##  部署指南\n  \n  \n**只有同时启动 go-cqhttp 和 HarukaBot 机器人才能正常运行.**\n  \n###  部署 go-cqhttp\n  \n  \n1. 下载 [`go-cqhttp`](https://github.com/Mrs4s/go-cqhttp/releases ) (Windows 用户选择 `windows-amd64.zip` 结尾).\n  \n2. 解压至一个空文件夹后, 双击启动, 此时文件夹内会生成一个 `config.json` 文件, 打开并编辑. \n以下折叠部分为参考 (中文部分记得替换): \n  \n<details>\n<summary>config.json 设置参考</summary>\n  \n```json\n{\n\t"uin": 机器人QQ号,\n\t"password": "QQ密码",\n\t"encrypt_password": false,\n\t"password_encrypted": "",\n\t"enable_db": true,\n\t"access_token": "",\n\t"relogin": {\n\t\t"enabled": true,\n\t\t"relogin_delay": 3,\n\t\t"max_relogin_times": 0\n\t},\n\t"_rate_limit": {\n\t\t"enabled": false,\n\t\t"frequency": 1,\n\t\t"bucket_size": 1\n\t},\n\t"ignore_invalid_cqcode": false,\n\t"force_fragmented": false,\n\t"heartbeat_interval": 0,\n\t"http_config": {\n\t\t"enabled": false,\n\t\t"host": "0.0.0.0",\n\t\t"port": 5700,\n\t\t"timeout": 0,\n\t\t"post_urls": {}\n\t},\n\t"ws_config": {\n\t\t"enabled": false,\n\t\t"host": "0.0.0.0",\n\t\t"port": 6700\n\t},\n\t"ws_reverse_servers": [\n\t\t{\n\t\t\t"enabled": true,\n\t\t\t"reverse_url": "ws://127.0.0.1:8080/cqhttp/ws",\n\t\t\t"reverse_api_url": "",\n\t\t\t"reverse_event_url": "",\n\t\t\t"reverse_reconnect_interval": 3000\n\t\t}\n\t],\n\t"post_message_format": "string",\n\t"debug": false,\n\t"log_level": "",\n\t"web_ui": {\n\t\t"enabled": false,\n\t\t"host": "127.0.0.1",\n\t\t"web_ui_port": 9999,\n\t\t"web_input": false\n\t}\n}\n```\n</details>\n  \n</br>\n  \n3. 编辑完重启 `go-cqhttp.exe`, 跟着提示完成安全验证即可\n  \n###  部署 HarukaBot\n  \n  \n####  方法一 手动安装 (较为复杂全平台通用)\n  \n  \n1. 安装 [Python3.7+](https://www.python.org/downloads/release/python-386/ ) (安装的时候一定要[**勾选 "Add Python 3.x to PATH"**](https://www.liaoxuefeng.com/wiki/1016959663602400/1016959856222624 ) )\n  \n2. 克隆 or [下载](https://github.com/SK-415/HarukaBot/releases ) HarukaBot 源码到本地\n  \n3. 在源码根目录打开命令提示符 (对着文件夹内, 按住 shift 同时鼠标右键 -> 在此处打开 Powershell 窗口)\n  \n3. 输入 `pip install -r requirements.txt` 安装依赖\n  \n4. 输入 `python bot.py` 启动 HarukaBot\n  \n> 以后每次启动只需重复 3, 5 步骤\n  \n####  方法二 插件广场安装 (适用于 `NoneBot2` 用户)\n  \n  \n- 前往[插件广场](https://v2.nonebot.dev/plugin-store.html ), 找到 `HarukaBot`, 点击下方 `COPY NB-CLI COMMAND` 复制安装指令\n  \n- 在自己的 `NoneBot2` 实例根目录运行安装指令\n  \n- 在 `bot.py` 中添加 `nonebot.load_plugin("haruka_bot")`\n  \n- (可选) 在 `.env.prod` 或 `.env.dev` 中添加 `HARUKA_DIR=".\\\\data\\\\"`, 你也可以将 `.\\\\data\\\\` 改成任何其他路径. \n  > 如果不添加, HarukaBot 会将配置文件保存于其安装包位置的 `data` 文件夹下, 不推荐.\n  \n- 完成后重启 `NoneBot2` 实例即可使用\n  \n##  支持作者\n  \n  \n点个小星星就是对我最好的支持. 感谢使用 HarukaBot. 也欢迎有能man对本项目pr.\n  ',
    'author': 'SK-415',
    'author_email': '2967923486@qq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/SK-415/HarukaBot',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
